use std::sync::Mutex;

use jni::{
  objects::{GlobalRef, JValueGen},
  JNIEnv,
};

use crate::component_name;

use super::Component;

pub struct DebugChat {
  buffer: Mutex<String>,
  object: GlobalRef,
}
impl DebugChat {
  pub(crate) fn new(component: GlobalRef) -> Self {
    Self {
      buffer: Default::default(),
      object: component,
    }
  }
}

impl Component for DebugChat {
  fn name() -> [u8; 0x30] {
    component_name!(b"chat")
  }

  fn memory_map_size(&self) -> u64 {
    1
  }

  fn write(&self, env: &mut JNIEnv, _: u64, data: &[u8]) {
    let mut string = self.buffer.lock().unwrap();
    if data[0] == 0 {
      let jni_string = env.new_string(string.drain(..).collect::<String>()).expect("failed to create string");
      env
        .call_method(
          self.object.clone(),
          "sendChatMessage",
          "(Ljava_lang_String;)V",
          &[JValueGen::Object(&jni_string)],
        )
        .expect("failed to call...");
      return;
    }
    string.push(data[0].into());
  }
}
