use core::CoreComponent;

use debug_chat::DebugChat;
use jni::JNIEnv;

pub mod core;
pub mod debug_chat;
pub mod interrupt_controller;
pub mod storage;

pub trait Component: Sized {
  fn name() -> [u8; 0x30];
  fn memory_map_size(&self) -> u64;

  fn write(&self, env: &mut JNIEnv, offset: u64, data: &[u8]) {
    let _ = (env, offset, data);
  }
  fn read(&self, env: &mut JNIEnv, offset: u64, data: &mut [u8]) {
    let _ = (env, offset, data);
  }
  fn has_interrupt(&self) -> bool {
    false
  }
}

pub enum AllComponent {
  Core(CoreComponent),
  DebugChat(DebugChat),
}

impl AllComponent {
  pub fn name(&self) -> [u8; 0x30] {
    match self {
      AllComponent::Core(_) => CoreComponent::name(),
      AllComponent::DebugChat(_) => DebugChat::name(),
    }
  }

  pub fn memory_map_size(&self) -> u64 {
    match self {
      AllComponent::Core(core_component) => core_component.memory_map_size(),
      AllComponent::DebugChat(debug_chat) => debug_chat.memory_map_size(),
    }
  }

  pub fn write(&self, env: &mut JNIEnv, offset: u64, data: &[u8]) {
    match self {
      AllComponent::Core(core_component) => core_component.write(env, offset, data),
      AllComponent::DebugChat(debug_chat) => debug_chat.write(env, offset, data),
    }
  }

  pub fn read(&self, env: &mut JNIEnv, offset: u64, data: &mut [u8]) {
    match self {
      AllComponent::Core(core_component) => core_component.read(env, offset, data),
      AllComponent::DebugChat(debug_chat) => debug_chat.read(env, offset, data),
    }
  }

  pub fn has_interrupt(&self) -> bool {
    match self {
      AllComponent::Core(core_component) => core_component.has_interrupt(),
      AllComponent::DebugChat(debug_chat) => debug_chat.has_interrupt(),
    }
  }
}
