package ca.sanae.golemcomputers

import ca.sanae.golemcomputers.blocks.GolemBlock
import ca.sanae.golemcomputers.blocks.GolemBlockEntity
import ca.sanae.golemcomputers.computer.RustNative
import com.mojang.serialization.Codec
import net.fabricmc.api.ModInitializer
import net.fabricmc.fabric.api.`object`.builder.v1.block.entity.FabricBlockEntityTypeBuilder
import net.minecraft.block.Block
import net.minecraft.block.entity.BlockEntity
import net.minecraft.block.entity.BlockEntityType
import net.minecraft.component.ComponentType
import net.minecraft.network.codec.PacketCodecs
import net.minecraft.registry.Registries
import net.minecraft.registry.Registry
import net.minecraft.registry.RegistryKey
import net.minecraft.registry.RegistryKeys
import net.minecraft.text.Text
import net.minecraft.text.TextCodecs
import net.minecraft.util.Identifier


class GolemComputers : ModInitializer {
  companion object {
    const val MOD_ID = "golem_computers"

    fun id(path: String): Identifier {
      return Identifier.of(MOD_ID, path)
    }

    val GOLEM_BLOCK_KEY: RegistryKey<Block> = RegistryKey.of(RegistryKeys.BLOCK, id("golem"))
    val GOLEM_BLOCK = register(GOLEM_BLOCK_KEY, GolemBlock())
    val GOLEM_BLOCK_ENTITY = register(GOLEM_BLOCK_KEY, ::GolemBlockEntity, GOLEM_BLOCK)

    private fun <T : Block> register(key: RegistryKey<Block>, block: T): T {
      return Registry.register(Registries.BLOCK, key, block)
    }

    private fun <T> register(key: RegistryKey<ComponentType<*>>, componentType: ComponentType<T>): ComponentType<T> {
      return Registry.register(Registries.DATA_COMPONENT_TYPE, key, componentType)
    }

    private fun <T : BlockEntity?> register(
      key: RegistryKey<Block>,
      entityFactory: FabricBlockEntityTypeBuilder.Factory<out T>,
      vararg blocks: Block
    ): BlockEntityType<T> {
      return Registry.register(
        Registries.BLOCK_ENTITY_TYPE,
        key.value,
        FabricBlockEntityTypeBuilder.create(entityFactory, *blocks).build()
      )
    }
  }

  override fun onInitialize() {
    if (!RustNative.isInitialized()) {
      error("Rust binary failed to load!")
    }
  }
}
