package ca.sanae.golemcomputers.blocks

import ca.sanae.golemcomputers.GolemComputers
import com.mojang.serialization.MapCodec
import net.minecraft.block.BlockState
import net.minecraft.block.BlockWithEntity
import net.minecraft.block.CommandBlock
import net.minecraft.block.entity.BlockEntity
import net.minecraft.block.entity.BlockEntityTicker
import net.minecraft.block.entity.BlockEntityType
import net.minecraft.entity.player.PlayerEntity
import net.minecraft.server.command.SayCommand
import net.minecraft.sound.BlockSoundGroup
import net.minecraft.text.Text
import net.minecraft.util.ActionResult
import net.minecraft.util.hit.BlockHitResult
import net.minecraft.util.math.BlockPos
import net.minecraft.world.World

class GolemBlock :
  BlockWithEntity(
    Settings.create().registryKey(GolemComputers.GOLEM_BLOCK_KEY).sounds(BlockSoundGroup.METAL).strength(100.0f)
  ) {
  override fun getCodec(): MapCodec<out BlockWithEntity> {
    return createCodec(::GolemBlock::call)
  }

  override fun createBlockEntity(pos: BlockPos, state: BlockState): BlockEntity {
    return GolemBlockEntity(pos, state)
  }

  override fun <T : BlockEntity?> getTicker(
    world: World?,
    state: BlockState?,
    type: BlockEntityType<T>?
  ): BlockEntityTicker<T>? {
    return validateTicker(
      type,
      GolemComputers.GOLEM_BLOCK_ENTITY
    ) { _, _, _, golemBlockEntity -> golemBlockEntity.tick() }
  }

  override fun onUse(
    state: BlockState,
    world: World,
    pos: BlockPos,
    player: PlayerEntity,
    hit: BlockHitResult
  ): ActionResult {
    val blockEntity = world.getBlockEntity(pos)
    if (blockEntity !is GolemBlockEntity)
      return super.onUse(state, world, pos, player, hit)
    if (world.isClient()) {
      return ActionResult.SUCCESS
    }
    world.server!!.playerManager.broadcast(Text.literal("Hello world!"), false)
    blockEntity.incrementClicks();
//    player.sendMessage(Text.literal("You've clicked the block for the ${blockEntity.getClicks()}th time."), true);

    return ActionResult.SUCCESS_SERVER
  }
}