package ca.sanae.golemcomputers.computer

import ca.sanae.golemcomputers.computer.NativeProtobuf.ChatComponentInit
import ca.sanae.golemcomputers.computer.NativeProtobuf.ComponentInit
import ca.sanae.golemcomputers.computer.NativeProtobuf.ComputerInit
import ca.sanae.golemcomputers.computer.NativeProtobuf.CoreComponentInit
import ca.sanae.golemcomputers.computer.RustNative.cleaner
import ca.sanae.golemcomputers.computer.components.ChatComponent
import net.minecraft.world.World
import java.nio.ByteBuffer
import java.nio.channels.FileChannel
import kotlin.io.path.Path

class Computer private constructor(val world: World, init: ComputerInit, initObjects: Array<Any>) {
  @Suppress("unused")
  private val address: Long = 0

  init {
    new(init.toByteArray(), initObjects)
    cleaner.register(this) {
      free()
    }
  }

  private external fun new(toByteArray: ByteArray, initObjects: Array<Any>): Long
  private external fun free()
  external fun resetCore(core: Int, wakeCore: Boolean)
  external fun tick();

  class Builder(val world: World, rom: ByteBuffer, ramPages: UInt) {
    private val objects: ArrayList<Any> = ArrayList()
    private val builder: ComputerInit.Builder = ComputerInit.newBuilder()

    init {
      builder.setPrimaryRomIndex(addObject(rom))
      builder.setPrimaryRamPages(ramPages.toInt())
    }

    private fun addObject(value: Any): Int {
      val index = objects.size
      objects.add(value)
      return index
    }

    fun addCore(instructionsPerTick: Int): Builder {
      builder.addComponents(
        ComponentInit.newBuilder()
          .setCore(CoreComponentInit.newBuilder().setInstructionsPerTick(instructionsPerTick).build())
      )

      return this
    }

    fun addChat(chat: ChatComponent): Builder {
      builder.addComponents(
        ComponentInit.newBuilder().setChat(ChatComponentInit.newBuilder().setJavaIndex(addObject(chat)).build())
      )

      return this
    }

    fun build(): Computer {
      return Computer(world, builder.build(), objects.toArray())
    }
  }
}